@@ -17,6 +17,14 @@ ifnotfinite <- function(x, otherwise = .Machine$double.xmax) {
   ifelse(is.finite(x), x, sign(x) * otherwise)
 }
 
+warnbound <- function() { 
+  getOption("spatialwarnings.debug.fit_warn_on_bound", default = FALSE)
+}
+
+warnNA <- function() { 
+  getOption("spatialwarnings.debug.fit_warn_on_NA", default = FALSE)
+}
+
 # This is a safe version of nlm that returns a sensible result (NaNs) when 
 # the algorithm fails to converge. This can happen quite often when looking 
 # for pathological cases (e.g. fitting distribution based on few points in the 
@@ -32,8 +40,8 @@ optim_safe <- function(f, pars0,
       f(exp(pars))
     }
     pars0 <- log(pars0)
-    lower <- ifnotfinite(log(lower))
-    upper <- ifnotfinite(log(upper))
+    lower <- suppressWarnings(ifnotfinite(log(lower)))
+    upper <- suppressWarnings(ifnotfinite(log(upper)))
   } else { 
     optimf <- f
   }
@@ -204,23 +212,22 @@ pl_ll <- function(dat, expo, xmin) {
 #'
 #' @return A list containing at list the following components: 
 #'
-#' \itemize{ 
-#'    \item{type: }{The type of distribution fitted (as a character string)}
-#'    \item{method: }{The method used for the fit - here, maximum likelihood,
-#'                  'll'}
-#'    \item{ll: }{The negative log likelihood at the estimated parameter values}
-#'    \item{xmin: }{The value of xmin used for the fit}
-#'    \item{npars: }{The number of parameters of the distribution}
+#' \describe{ 
+#'    \item{\code{type}}{The type of distribution fitted (as a character string)}
+#'    \item{\code{method}}{The method used for the fit - here, maximum likelihood, 'll'}
+#'    \item{\code{ll}}{The log likelihood at the estimated parameter values}
+#'    \item{\code{xmin}}{The value of xmin used for the fit}
+#'    \item{\code{npars}}{The number of parameters of the distribution}
 #'  }
 #' 
-#' Additionnaly, this list may have one or more of the following parameters 
-#'   depending on the type of distribution that has been fitted: 
-#'   \itemize{ 
-#'     \item{plexpo: }{The exponent of the power-law}
-#'     \item{cutoff: }{The rate of truncation, for truncated power law and 
-#'                 exponential fits}
-#'     \item{meanlog: }{The mean of the lognormal distribution}
-#'     \item{sdlog: }{The s.d. of the lognormal distribution}
+#' Additionally, this list may have one or more of the following elements depending on 
+#'   the type of distribution that has been fitted: 
+#'   \describe{ 
+#'     \item{\code{plexpo}}{The exponent of the power-law}
+#'     \item{\code{cutoff}}{The rate of truncation, for truncated power law and 
+#'        exponential fits}
+#'     \item{\code{meanlog}}{The mean of the lognormal distribution}
+#'     \item{\code{sdlog}}{The s.d. of the lognormal distribution}
 #'   }
 #' 
 #' @details These functions will fit distributions to a set of values using 
@@ -241,21 +248,38 @@ pl_ll <- function(dat, expo, xmin) {
 #' 
 #' Please note that a best effort is made to have the fit converge, but 
 #'   it may sometimes fail when the parameters are far from their usual 
-#'   range. It is good practice to make sure the fits are sensible when 
-#'   convergence warnings are reported.
+#'   range, and numerical issues may occur. It is good practice to make 
+#'   sure the fits are sensible when convergence warnings are reported.
 #' 
 #' For reference, the shape of the distributions is as follow: 
 #' 
-#' \itemize{
-#'   \item{power-law }{\eqn{x^{-a}}{x^(-a)} where a is the power-law exponent}
-#'   \item{exponential }{\eqn{exp(-bx)}{exp(-bx)} where b is the truncation rate
-#'           of the exponential } 
-#'   \item{truncated power-law }{\eqn{x^{-a}exp(-bx)}{x^(-a)exp(-bx)} where a
+#' \describe{
+#'   \item{power-law}{\eqn{x^{-a}}{x^(-a)} where a is the power-law exponent}
+#'   \item{exponential}{\eqn{exp(-bx)}{exp(-bx)} where b is the truncation rate
+#'           of the exponential}
+#'   \item{truncated power-law}{\eqn{x^{-a}exp(-bx)}{x^(-a)exp(-bx)} where a
 #'     and b are the exponent of the power law and the rate of truncation}
 #' }
 #' 
 #' The lognormal form follows the \link[=dlnorm]{standard definition}.
 #' 
+#' The following global options can be used to change the behavior of fitting functions 
+#'   and/or produce more verbose output: 
+#' \describe{ 
+#'   \item{spatialwarnings.constants.reltol}{the relative tolerance to use to compute 
+#'     the power-law normalizing constant
+#'     \deqn{sum_{k=1}^{\infty} x^{ak}e^{-bk}}{sum( x^(ak)exp(-bk)) for k in 1:Inf}. 
+#'     Increase to increase the precision of this constant, which can be useful in some 
+#'     cases, typically with large sample sizes. Default is 1e-8.}
+#'   \item{spatialwarnings.constants.maxit}{the maximum number of iterations to compute 
+#'     the normalizing constant of a truncated power-law. Increase if you get a warning 
+#'     that the relative tolerance level (defined above) was not reached. Default is 1e8}
+#'   \item{spatialwarnings.debug.fit_warn_on_bound}{logical value. Warn if the fit is 
+#'     at the boundary of the valid range for distribution parameter}
+#'   \item{spatialwarnings.debug.fit_warn_on_NA}{logical value. Warn if the returned fit 
+#'     has \code{NA}/\code{NaN} parameters}
+#' }
+#' 
 #' @seealso \code{\link{patchdistr_sews}}, \code{\link{xmin_estim}}
 #' 
 #' @references
@@ -270,7 +294,7 @@ pl_ll <- function(dat, expo, xmin) {
 #' exp_fit(patchsizes(forestgap[[8]]))
 #'  
 #' # Use the estimated parameters as an indicator function
-#' \dontrun{
+#' \donttest{
 #' 
 #' get_truncation <- function(mat) { 
 #'    c(exp_cutoff = exp_fit(patchsizes(mat))$cutoff)
@@ -300,6 +324,7 @@ pl_fit <- function(dat, xmin = 1) {
     }
   }
   
+  
   est <- optim_safe(negll, expo_estim, 
                     lower = PLMIN, upper = PLMAX)
   
@@ -309,6 +334,16 @@ pl_fit <- function(dat, xmin = 1) {
                  ll = - est[['value']],
                  xmin = xmin,
                  npars = 1)
+  
+  if ( warnNA() && is.na(result[["plexpo"]]) ) { 
+    warning("Fitting of PL returned NA")
+    return(result)
+  }
+  
+  if ( warnbound() && any(abs(result[["plexpo"]] - c(PLMIN, PLMAX)) < 1e-8) ) { 
+    warning("Estimated PL exponent is on the boundary of the valid range")
+  }
+  
   return(result)
 }
 
@@ -339,7 +374,7 @@ pl_fit <- function(dat, xmin = 1) {
 #' 
 #' @examples 
 #' 
-#' \dontrun{ 
+#' \donttest{ 
 #' psd <- patchsizes(forestgap[[5]])
 #' xmin_estim(psd)
 #' }
@@ -473,6 +508,11 @@ exp_fit <- function(dat, xmin = 1) {
                  cutoff = est[['par']], 
                  ll = - est[["value"]],
                  npars = 1)
+  
+  if ( warnbound() && any(abs(result[["cutoff"]] - c(EXPMIN, EXPMAX)) < 1e-8) ) { 
+    warning("Estimated EXP exponent is on the boundary of the valid range")
+  }
+  
   return(result)
 }
 
@@ -543,7 +583,13 @@ lnorm_fit <- function(dat, xmin = 1) {
 # ---------------------------------------
 
 tplnorm <- function(expo, rate, xmin) { 
-  tplinfsum(expo, rate, xmin)
+  
+  maxit <- getOption("spatialwarnings.constants.maxit", default = 1e8L)
+  reltol <- getOption("spatialwarnings.constants.reltol", default = 1e-8)
+  
+  a <- tplinfsum(expo, rate, xmin, maxit, reltol)
+  
+  a
 }
 
 # P(x=k)
@@ -585,7 +631,7 @@ tpl_ll <- function(x, expo, rate, xmin, approximate = FALSE) {
 tpl_fit <- function(dat, xmin = 1) { 
   
   negll <- function(pars) { 
-    - tpl_ll(dat, pars[1], pars[2], xmin)
+    - tpl_ll(dat, pars[1], pars[2], xmin) 
   }
   
   # Initialize and find minimum
@@ -594,16 +640,41 @@ tpl_fit <- function(dat, xmin = 1) {
   # Do a line search over the cutoff to find a minimum, starting from zero 
   # up to 100
   is <- seq(0, 100, length = 128)
+  is <- 10^seq(-7, 2, l = 128)
   lls <- unlist(lapply(is, function(i) { 
     negll(c(expo0, i))
   }))
-  expmrate0 <- is[which.min(lls)]
+  llmin <- min(lls[abs(lls) != .Machine$double.xmax & is.finite(lls)])
+  expmrate0 <- is[which(lls == llmin)]
+  
+  # If multiple cutoffs produce the same ll, then use the lowest one 
+  if ( length(expmrate0) > 1 ) { 
+    expmrate0 <- expmrate0[1]
+  }
+  
+  # Debug thing lls
+#   plot( log(spatialwarnings:::cumpsd(dat)) )
+#   vals <- ippl(dat, expo0, xmin = xmin)
+#   points(log(dat), log(vals), col = "red")
+#   vals <- iptpl(dat, expo0, rate = 0, xmin = xmin)
+#   points(log(dat), log(vals), col = "darkgreen")
+#   vals <- iptpl(dat, expo0, rate = expmrate0, xmin = xmin)
+#   points(log(dat), log(vals), col = "blue")
   
   pars0 <- c(expo0, expmrate0)
-  
   est <- optim_safe(negll, pars0, 
                     lower = c(TPL_EXPOMIN, TPL_RATEMIN), 
-                    upper = c(TPL_EXPOMAX, TPL_RATEMAX))
+                    upper = c(TPL_EXPOMAX, TPL_RATEMAX), 
+                    fit_on_logscale = FALSE)
+  
+  # For very small cutoffs, we may want to fit on log scale to get a descent estimate, 
+  # as otherwise there is too little variation in the cutoff for BFGS
+  if ( any(is.na(est[["par"]])) ) { 
+    est <- optim_safe(negll, pars0, 
+                      lower = c(TPL_EXPOMIN, TPL_RATEMIN), 
+                      upper = c(TPL_EXPOMAX, TPL_RATEMAX), 
+                      fit_on_logscale = TRUE)
+  }
   
   result <- list(type = 'tpl',
                  method = 'll', 
@@ -611,5 +682,26 @@ tpl_fit <- function(dat, xmin = 1) {
                  cutoff = est[['par']][2], 
                  ll = - est[["value"]],
                  npars = 2)
+  
+  if ( warnNA() && is.na(result[["plexpo"]]) ) { 
+    warning("Fitting of TPL returned NA/NaN")
+  }
+  
+  if ( is.na(result[["plexpo"]]) ) { 
+    return(result)
+  }
+  
+  if ( warnbound() && 
+       any(abs(result[["plexpo"]] - c(TPL_EXPOMIN, TPL_EXPOMAX)) < 1e-8) ) { 
+    warning("Estimated TPL exponent is on the boundary of the valid range")
+  }
+  
+  # We do not warn for reaching minimum exponent because this is quite common when 
+  # fitting truncated power laws to have a very small exponent
+  if ( warnbound() && 
+       any(abs(result[["cutoff"]] - c(TPL_RATEMIN, TPL_RATEMAX)) < 1e-8) ) { 
+    warning("Estimated TPL cutoff is on the boundary of the valid range")
+  }
+  
   return(result)
 }
